from sila2.framework.binary_transfer.binary_transfer_error import BinaryTransferError, BinaryTransferErrorType


class BinaryDownloadFailed(BinaryTransferError):
    def __init__(self, message: str):
        super().__init__(BinaryTransferErrorType.BINARY_DOWNLOAD_FAILED, message)
